@@ -1,5 +1,5 @@
 use crate::*;
-use std::convert::TryInto;
+use std::fmt::Debug;
 use std::ops::{AddAssign, Mul};
 
 /// A trait for matrix multiplication.
@@ -19,358 +19,278 @@ pub trait Matmul<T> {
 // - M: Columns of `self` and rows of `other`
 // - N: Columns of `other` and rows of `Self::Output`
 
+// Inline allows some specific optimization when the dimensions are const?
+/// `c = a @ b`
+#[inline]
+fn matmul<T: Debug + Mul<Output = T> + AddAssign + Copy + Debug>(
+    (l_len, n_len, m_len): (usize, usize, usize),
+    (a, b, c): (&[T], &[T], &mut [T]),
+) {
+    debug_assert_eq!(a.len(), l_len * m_len);
+    debug_assert_eq!(b.len(), m_len * n_len);
+    debug_assert_eq!(c.len(), l_len * n_len);
+
+    for l in 0..l_len {
+        for n in 0..n_len {
+            for m in 0..m_len {
+                let (i, j, k) = (l * n_len + n, l * m_len + m, m * n_len + n);
+                c[i] += a[j] * b[k];
+                // c[l][n] += a[l][m] * b[m][n];
+            }
+        }
+    }
+}
+
 // MatrixDxD
 // --------------------------------------------------
-impl<T: Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
     Matmul<MatrixSxS<T, M, N>> for MatrixDxD<T>
+where
+    [(); M * N]:,
 {
     type Output = MatrixDxS<T, N>;
     fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
-        assert_eq!(self.data[0].len(), M, "Non-matching columns to rows");
+        assert_eq!(self.columns, M, "Non-matching columns to rows");
 
-        let temp_l = self.data.len();
-        let mut data = vec![[Default::default(); N]; temp_l];
-        for l in 0..temp_l {
-            for n in 0..N {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let l_len = self.rows;
+        let mut data = vec![Default::default(); N * l_len];
+        matmul((l_len, N, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, rows: l_len }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T>, const N: usize> Matmul<MatrixDxS<T, N>>
-    for MatrixDxD<T>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const N: usize>
+    Matmul<MatrixDxS<T, N>> for MatrixDxD<T>
 {
     type Output = MatrixDxS<T, N>;
     fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
-        assert_eq!(
-            self.data[0].len(),
-            other.data.len(),
-            "Non-matching columns to rows"
-        );
+        assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
-        let temp_l = self.data.len();
-        let temp_m = self.data[0].len();
-        let mut data = vec![[Default::default(); N]; temp_l];
-        for l in 0..temp_l {
-            for n in 0..N {
-                for m in 0..temp_m {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let (l, m) = (self.rows, self.columns);
+        let mut data = vec![Default::default(); N * l];
+        matmul((l, N, m), (&self.data, &other.data, &mut data));
+        Self::Output { data, rows: l }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
     Matmul<MatrixSxD<T, M>> for MatrixDxD<T>
 {
     type Output = MatrixDxD<T>;
     fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
-        let temp_l = self.data.len();
-        let temp_n = other.data[0].len();
-        let mut data = Vec::with_capacity(temp_l);
-        for l in 0..temp_l {
-            data.push(vec![Default::default(); temp_n]);
-            for n in 0..temp_n {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
+        let (l, n) = (self.rows, other.columns);
+        let mut data = vec![Default::default(); l * n];
+        matmul((l, n, M), (&self.data, &other.data, &mut data));
+        Self::Output {
+            data,
+            rows: l,
+            columns: n,
         }
-        Self::Output { data }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug> Matmul<MatrixDxD<T>>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug> Matmul<MatrixDxD<T>>
     for MatrixDxD<T>
 {
     type Output = MatrixDxD<T>;
     fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
-        assert_eq!(
-            self.data[0].len(),
-            other.data.len(),
-            "Non-matching columns to rows"
-        );
+        assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
-        let temp_l = self.data.len();
-        let temp_m = self.data[0].len();
-        let temp_n = other.data[0].len();
-        let mut data = Vec::with_capacity(temp_l);
-        for l in 0..temp_l {
-            data.push(vec![Default::default(); temp_n]);
-            for n in 0..temp_n {
-                for m in 0..temp_m {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
+        let (l, m, n) = (self.rows, self.columns, other.columns);
+        let mut data = vec![Default::default(); l * n];
+        matmul((l, n, m), (&self.data, &other.data, &mut data));
+        Self::Output {
+            data,
+            rows: l,
+            columns: n,
         }
-        Self::Output { data }
     }
 }
 // MatrixDxS
 // --------------------------------------------------
-impl<T: Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
     Matmul<MatrixSxS<T, M, N>> for MatrixDxS<T, M>
+where
+    [(); M * N]:,
 {
     type Output = MatrixDxS<T, N>;
     fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
-        let temp_l = self.data.len();
-        let mut data = vec![[Default::default(); N]; temp_l];
-        for l in 0..temp_l {
-            for n in 0..N {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let l = self.rows;
+        let mut data = vec![Default::default(); l * N];
+        matmul((l, N, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, rows: l }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const M: usize, const N: usize>
     Matmul<MatrixDxS<T, N>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxS<T, N>;
     fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
-        let temp_l = self.data.len();
-        let mut data = vec![[Default::default(); N]; temp_l];
-        for l in 0..temp_l {
-            for n in 0..N {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let l = self.rows;
+        let mut data = vec![Default::default(); l * N];
+        matmul((l, N, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, rows: l }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
     Matmul<MatrixSxD<T, M>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxD<T>;
     fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
-        let temp_l = self.data.len();
-        let temp_n = other.data[0].len();
-        let mut data = Vec::with_capacity(temp_l);
-        for l in 0..temp_l {
-            data.push(vec![Default::default(); temp_n]);
-            for n in 0..temp_n {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
+        let (l, n) = (self.rows, other.columns);
+        let mut data = vec![Default::default(); l * n];
+        matmul((l, n, M), (&self.data, &other.data, &mut data));
+        Self::Output {
+            data,
+            rows: l,
+            columns: n,
         }
-        Self::Output { data }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const M: usize>
     Matmul<MatrixDxD<T>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxD<T>;
     fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
-        assert_eq!(M, other.data.len(), "Non-matching columns to rows");
+        assert_eq!(M, other.rows, "Non-matching columns to rows");
 
-        let temp_l = self.data.len();
-        let temp_n = other.data[0].len();
-        let mut data = Vec::with_capacity(temp_l);
-        for l in 0..temp_l {
-            data.push(vec![Default::default(); temp_n]);
-            for n in 0..temp_n {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
+        let (l, n) = (self.rows, other.columns);
+        let mut data = vec![Default::default(); l * n];
+        matmul((l, n, M), (&self.data, &other.data, &mut data));
+        Self::Output {
+            data,
+            rows: l,
+            columns: n,
         }
-        Self::Output { data }
     }
 }
 // MatrixSxD
 // --------------------------------------------------
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T>,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T>,
         const L: usize,
         const M: usize,
         const N: usize,
     > Matmul<MatrixSxS<T, M, N>> for MatrixSxD<T, L>
+where
+    [(); L * N]:,
+    [(); M * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
     fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
-        let mut data = [[Default::default(); N]; L];
-        for l in 0..L {
-            for n in 0..N {
-                // data[l][n] = self.data[l].iter().zip(other.data[..][n]).map(|(a,b)|*a*b).sum();
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
+        let mut data = [Default::default(); L * N];
+        matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T>, const L: usize, const N: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const L: usize, const N: usize>
     Matmul<MatrixDxS<T, N>> for MatrixSxD<T, L>
+where
+    [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
     fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
-        assert_eq!(
-            self.data[0].len(),
-            other.data.len(),
-            "Non-matching columns to rows"
-        );
-        let mut data = [[Default::default(); N]; L];
-        for l in 0..L {
-            for n in 0..N {
-                for m in 0..self.data[0].len() {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
+        assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
+
+        let m = self.columns;
+        let mut data = [Default::default(); L * N];
+        matmul((L, N, m), (&self.data, &other.data, &mut data));
         Self::Output { data }
     }
 }
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
         const L: usize,
         const M: usize,
     > Matmul<MatrixSxD<T, M>> for MatrixSxD<T, L>
 {
     type Output = MatrixSxD<T, L>;
     fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
-        let n_temp = other.data[0].len();
-        let mut data: [Vec<T>; L] = (0..L)
-            .map(|_| vec![Default::default(); n_temp])
-            .collect::<Vec<_>>()
-            .try_into()
-            .unwrap();
-        for l in 0..L {
-            for n in 0..n_temp {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let n = other.columns;
+        let mut data = vec![Default::default(); L * n];
+        matmul((L, n, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, columns: n }
     }
 }
-impl<T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const L: usize>
+impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug, const L: usize>
     Matmul<MatrixDxD<T>> for MatrixSxD<T, L>
 {
     type Output = MatrixSxD<T, L>;
     fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
-        assert_eq!(
-            self.data[0].len(),
-            other.data.len(),
-            "Non-matching columns to rows"
-        );
+        assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
-        let n_temp = other.data[0].len();
-        let mut data: [Vec<T>; L] = (0..L)
-            .map(|_| vec![Default::default(); n_temp])
-            .collect::<Vec<_>>()
-            .try_into()
-            .unwrap();
-        for l in 0..L {
-            for n in 0..n_temp {
-                for m in 0..self.data[0].len() {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let (n, m) = (other.columns, self.columns);
+        let mut data = vec![Default::default(); L * n];
+        matmul((L, n, m), (&self.data, &other.data, &mut data));
+        Self::Output { data, columns: n }
     }
 }
 // MatrixSxS
 // --------------------------------------------------
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T>,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T>,
         const L: usize,
         const M: usize,
         const N: usize,
     > Matmul<MatrixSxS<T, M, N>> for MatrixSxS<T, L, M>
+where
+    [(); L * M]:,
+    [(); M * N]:,
+    [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
     fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
-        let mut data = [[Default::default(); N]; L];
-        for l in 0..L {
-            for n in 0..N {
-                // data[l][n] = self.data[l].iter().zip(other.data[..][n]).map(|(a,b)|*a*b).sum();
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
+        let mut data = [Default::default(); L * N];
+        matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
     }
 }
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T>,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T>,
         const L: usize,
         const M: usize,
         const N: usize,
     > Matmul<MatrixDxS<T, N>> for MatrixSxS<T, L, M>
+where
+    [(); L * M]:,
+    [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
     fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
-        let mut data = [[Default::default(); N]; L];
-        for l in 0..L {
-            for n in 0..N {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
+        let mut data = [Default::default(); L * N];
+        matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
     }
 }
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
         const L: usize,
         const M: usize,
     > Matmul<MatrixSxD<T, M>> for MatrixSxS<T, L, M>
+where
+    [(); L * M]:,
 {
     type Output = MatrixSxD<T, L>;
     fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
-        let n_temp = other.data[0].len();
-        let mut data: [Vec<T>; L] = (0..L)
-            .map(|_| vec![Default::default(); n_temp])
-            .collect::<Vec<_>>()
-            .try_into()
-            .unwrap();
-        for l in 0..L {
-            for n in 0..n_temp {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let n = other.columns;
+        let mut data = vec![Default::default(); L * n];
+        matmul((L, n, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, columns: n }
     }
 }
 impl<
-        T: Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
+        T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
         const L: usize,
         const M: usize,
     > Matmul<MatrixDxD<T>> for MatrixSxS<T, L, M>
+where
+    [(); L * M]:,
 {
     type Output = MatrixSxD<T, L>;
     fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
-        assert_eq!(M, other.data.len(), "Non-matching columns to rows");
+        assert_eq!(M, other.rows, "Non-matching columns to rows");
 
-        let n_temp = other.data[0].len();
-        let mut data: [Vec<T>; L] = (0..L)
-            .map(|_| vec![Default::default(); n_temp])
-            .collect::<Vec<_>>()
-            .try_into()
-            .unwrap();
-        for l in 0..L {
-            for n in 0..n_temp {
-                for m in 0..M {
-                    data[l][n] += self.data[l][m] * other.data[m][n];
-                }
-            }
-        }
-        Self::Output { data }
+        let n = other.columns;
+        let mut data = vec![Default::default(); L * n];
+        matmul((L, n, M), (&self.data, &other.data, &mut data));
+        Self::Output { data, columns: n }
     }
 }
 // Tests
